import numpy as np
import math
from scipy.signal import argrelextrema

from AnyQt.QtCore import Qt
import pyqtgraph as pg

from orangecontrib.timeseries.widgets.owperiodbase import OWPeriodBase
from orangewidget.settings import Setting
from orangewidget.utils.widgetpreview import WidgetPreview

from Orange.widgets import gui
from orangecontrib.timeseries import Timeseries

#from orangecontrib.timeseries import (
#    Timeseries, autocorrelation, partial_autocorrelation)

def _significant_acf(corr, has_confint):
    if has_confint:
        corr, confint = corr

    periods = argrelextrema(np.abs(corr), np.greater, order=3)[0]
    num = (len(corr))
    periods = np.arange(num)
    corr = corr[periods]
    if has_confint:
        confint = confint[periods]

    result = np.column_stack((periods, corr))
    if has_confint:
        result = (result, np.column_stack((periods, confint)))
    return result

def autocorrelation(x, *args, nlags=None, fft=True, **kwargs):
    """
    Return autocorrelation function of signal `x`.

    Parameters
    ----------
    x: array_like
        A 1D signal.
    nlags: int
        The number of lags to calculate the correlation for (default .9*len(x))
    fft:  bool
        Compute the ACF via FFT.
    args, kwargs
        As accepted by `statsmodels.tsa.stattools.acf`.

    Returns
    -------
    acf: array
        Autocorrelation function.
    confint: array, optional
        Confidence intervals if alpha kwarg provided.
    """
    from statsmodels.tsa.stattools import acf
    if nlags == 0:
#        nlags = int(.9 * len(x))
        nlags = int(10 * math.log10(len(x)))
    corr = acf(x, *args, nlags=nlags, fft=fft, **kwargs)
    return _significant_acf(corr, kwargs.get('alpha'))


def partial_autocorrelation(x, *args, nlags=None, method='ldb', **kwargs):
    """
    Return partial autocorrelation function (PACF) of signal `x`.

    Parameters
    ----------
    x: array_like
        A 1D signal.
    nlags: int
        The number of lags to calculate the correlation for
        (default: min(len(x)//2 - 1, len(x) - 1))
    args, kwargs
        As accepted by `statsmodels.tsa.stattools.pacf`.

    Returns
    -------
    acf: array
        Partioal autocorrelation function.
    confint : optional
        As returned by `statsmodels.tsa.stattools.pacf`.
    """
    from statsmodels.tsa.stattools import pacf
    if nlags == 0:
        nlags = min(len(x)//2 - 1, len(x) - 1)
    corr = pacf(x, *args, nlags=nlags, method=method, **kwargs)
    return _significant_acf(corr, kwargs.get('alpha'))


class OWCorrelogram(OWPeriodBase):
    # TODO: allow computing cross-correlation of two distinct series
    name = 'Correlogram_mod'
    description = "Visualize variables' auto-correlation."
    icon = 'icons/Correlogram.svg'
    priority = 110
    lags = Setting(0)
    use_pacf = Setting(False)
    use_confint = Setting(True)

    yrange = (-1, 1)

    def __init__(self):
        super().__init__()
        gui.separator(self.controlArea)
        gui.spin(widget=self.controlArea, master=self, value='lags', minv=0, maxv=300, step=1, label="Lags:",
                 alignment=Qt.AlignRight, controlWidth=80, callback=self.replot)
        gui.checkBox(self.controlArea, self, 'use_pacf',
                     label='Compute partial auto-correlation',
                     callback=self.replot)
        gui.checkBox(self.controlArea, self, 'use_confint',
                     label='Plot 95% significance interval',
                     callback=self.replot)


    def acf(self, attr, lags,pacf, confint):
        key = (attr, pacf,lags, confint)
        if key not in self._cached:
            x = self.data.interp(attr).ravel()
            func = partial_autocorrelation if pacf else autocorrelation
            self._cached[key] = func(x, nlags=self.lags,alpha=.05 if confint else None)
        return self._cached[key]

    def replot(self):
        self.plot.clear()
        if not self.selection:
            return

        self.plot_widget.addItem(pg.InfiniteLine(0, 0, pen=pg.mkPen(0., width=2)))

        palette = self.get_palette()
        for i, attr in enumerate(self.selection):
            color = palette.value_to_qcolor(i)
            x, acf = np.array(self.acf(attr, self.lags,self.use_pacf, False)).T
            x = np.repeat(x, 2)
            y = np.vstack((np.zeros(len(acf)), acf)).T.flatten()
            item = pg.PlotCurveItem(
                x=x, y=y, connect="pairs", antialias=True,
                pen=pg.mkPen(color, width=5))
            self.plot_widget.addItem(item)

            if self.use_confint:
                # Confidence intervals, from:
                # https://www.mathworks.com/help/econ/autocorrelation-and-partial-autocorrelation.html
                # https://www.mathworks.com/help/signal/ug/confidence-intervals-for-sample-autocorrelation.html
                se = np.sqrt((1 + 2 * (acf ** 2).sum()) / len(self.data))
                #std = 1.96 * se
                std = 1.96 * se / np.sqrt(len(acf))
                color = palette.value_to_qcolor(i + 1)
                pen = pg.mkPen(color, width=2, style=Qt.DashLine)
                self.plot_widget.addItem(pg.InfiniteLine(std, 0, pen=pen))
                self.plot_widget.addItem(pg.InfiniteLine(-std, 0, pen=pen))


if __name__ == "__main__":
    WidgetPreview(OWCorrelogram).run(
        Timeseries.from_file("airpassengers")
    )



import Orange
from orangewidget import settings
from orangewidget.widget import OWBaseWidget, Input, Message
from orangewidget import gui
from Orange.widgets.widget import Output, Msg
from Orange.data import Table, Domain, ContinuousVariable, DiscreteVariable, TimeVariable, StringVariable
from AnyQt.QtWidgets import QTextEdit

# Otros imports
from dataclasses import field
import mysql.connector
from mysql.connector import FieldType

class MySqlOrange(OWBaseWidget):
    # Nombre del widget como se verá en el lienzo
    name = 'MySql Query'
    icon = 'icons/mysql-orange.svg'
    keywords = ["widget", "tools"]
    # category = 'Data'
    category = 'Tools'
    
    # Parámetros del widget
    host = settings.Setting('kaos.unizar.es')
    port = settings.Setting(31766)
    user = settings.Setting('aire')
    password = settings.Setting('CirceEolicos')
    database = settings.Setting('curva')
#    query = settings.Setting('select TIMESTAMP, RecNbr, Vel1_Avg, Vel1_Std, Vel1_Max, Vel1_Min from datalogger;')
    query = settings.Setting('select id,t,N_datos,Pot,Pot_std,Pot_max,Pot_min,VT1,VT1_std,VT1_max,VT1_min,VT2,VT2_std,VT2_max,VT2_min,Dir,Temp,Temp_std,Temp_max,Temp_min, Pres,Pres_std,Pres_max,Pres_min,Hum, Hum_std,Hum_min,Hum_max,Disp1,Disp2 from curva_promediada;')

    class Outputs:
        data = Output('Data', Table)

    def open_query(self):
    # clase para guardar los campos de la consulta y sus tipos
        class Campo:
            def __init__(self, nombre = '', tipo = -1):
                self.nombre = nombre
                self.tipo = tipo
        
        list_global = [] # lista final. Contendrá los datos de salida
        list_temp = [] # lista temporal, usada sólo para conversión
        list_domain = [] # lista en la que se guardarán los tipos de datos Orange que corresponden a cada columna
        list_campos = [] # lista de campos y tipo de datos de cada campo

        self.conn = mysql.connector.connect(user=self.user, password=self.password, host=self.host, port=self.port, db=self.database)
        cur = self.conn.cursor() 
        cur.execute(self.query)
        rows = cur.fetchall()

        # guardar los nombres y tipos de campo
        for desc in cur.description:
            colname = desc[0]
            coltype = desc[1]
            campo = Campo(colname, FieldType.get_info(coltype))
            list_campos.append(campo)

        self.conn.close()

        # Además hay que especificar el tipo de valor para el domain para cada fila
        single_row = rows[0]
        for i in range(len(single_row)):
            # se asigna el tipo de datos de la salida a list_domain según los valores de la lista list_campos
            # y se van añadiendo los valores devueltos por MySql a la lista list_temp
            nombre_campo = list_campos[i].nombre
            tipo_datos = list_campos[i].tipo
            if tipo_datos in ['VAR_STRING', 'STRING', 'BLOB','SET'] :
                list_domain.append(StringVariable(nombre_campo))
            elif tipo_datos in ['DATETIME', 'TIMESTAMP', 'TIME', 'DATE']:
                list_domain.append(TimeVariable(nombre_campo))
            elif tipo_datos in ['DECIMAL', 'NEWDECIMAL', 'LONG', 'INT', 'FLOAT', 'DOUBLE','BIT', 'YEAR']:
                list_domain.append(ContinuousVariable(nombre_campo))
            else:  
                raise Exception('Error: Tipo de valor no reconocido: ' + tipo_datos)

        # A continuación las filas (recibidas como tuplas) se convierten a una lista, que
        # es el formato que debe tener el output del witget

        # para cada fila recibida de la bd
        for single_row in rows :
            # para cada campo en la fila
            for i in range(len(single_row)):
                # se asigna el tipo de datos de la salida a list_domain según los valores de la lista list_campos
                # y se van añadiendo los valores devueltos por MySql a la lista list_temp
                valor_guardar = single_row[i]
                nombre_campo = list_campos[i].nombre
                if tipo_datos in ['VAR_STRING', 'STRING','BLOB','SET'] :
                    # list_temp.append(str(valor_guardar))
                    list_temp.append(str(valor_guardar))
                elif tipo_datos in ['DATETIME', 'TIMESTAMP', 'TIME', 'DATE']:
                    # list_temp.append(valor_guardar)
                    list_temp.append(str(valor_guardar))
                elif tipo_datos in ['DECIMAL', 'NEWDECIMAL', 'LONG', 'INT', 'FLOAT', 'DOUBLE','BIT', 'YEAR']:
                    # list_temp.append(int(valor_guardar))
                    list_temp.append(str(valor_guardar))
                    # list_temp.append(valor_guardar)
                else:  
                    print(tipo_datos)
                    raise Exception('Error: Tipo de valor no reconocido' + tipo_datos)
            list_global.append(list_temp)
            list_temp = []

        # data = self.get_table()
        # self.data_desc_table = data
        # self.Outputs.data.send(data)
        try:
            dominio = Domain(list_domain)
            out_data = Table.from_list(dominio, list_global)
            self.Outputs.data.send(out_data)
            # self.message_bar.clear()
            # self.warning('Query OK')
        except Exception as err:
            self.error('Error: ' + err)

    # class Outputs:
    #     data = Output('Data', Table, doc='Attribute-valued dataset read from the query.')

    want_main_area = False

    def __init__(self):
        super().__init__()

        self.data = None
        # self.proportions = 30
        box = gui.widgetBox(self.controlArea, 'Parameters')
        gui.lineEdit(box, self, 'host', 'Host',  valueType=str)
        gui.lineEdit(box, self, 'port', 'Port',  valueType=int)
        gui.lineEdit(box, self, 'user', 'User',  valueType=str) # TODO: password
        pw=gui.lineEdit(box, self, 'password', 'Password',  valueType=str)
        pw.setEchoMode(pw.Password)
        pw.show()
        gui.lineEdit(box, self, 'database', 'Database',  valueType=str)
        gui.lineEdit(box, self, 'query', 'Query' , valueType=str)
        gui.button(box, self, 'Ok', callback=self.ok_button)


    # @Inputs.data
    # def set_data(self, data):
    #     self.Data = data

    def commit(self):
        self.open_query()

    def ok_button(self):
        self.open_query()

        return
              
    
if __name__ == '__main__':
    from Orange.widgets.utils.widgetpreview import WidgetPreview  # since Orange 3.20.0
    WidgetPreview(MySqlOrange).run()


import threading
import textwrap
from typing import List, Dict, Any, Optional, Tuple
from Orange.data import Table, Domain, StringVariable, DiscreteVariable, ContinuousVariable
import Orange.data
try:
    from Orange.data.sql.table import SqlTable
except ImportError:
    SqlTable = None
from AnyQt.QtCore import Qt
from typing import List, Dict, Any, Optional, Tuple

from Orange.widgets import widget, gui
from Orange.widgets.widget import OWWidget, Msg, Input
from Orange.widgets.utils.localization import pl
from Orange.widgets.utils.widgetpreview import WidgetPreview
from Orange.widgets.widget import Input
from Orange.widgets.utils.itemmodels import DomainModel
from Orange.widgets import settings
from Orange.widgets.settings import ContextSetting, ContextHandler, Context
from Orange.data.pandas_compat import table_from_frame,table_to_frame

from orangecontrib.timeseries import Timeseries
from orangecontrib.timeseries import Timeseries as TS
from statsmodels.tsa.stattools import adfuller, kpss

class StationarityContextHandler(ContextHandler):
    """
    Context handler of Stationarity. The specifics of this widget is
    that `attrs` variable is list of lists and it is not handled with
    the DomainContextHandler.
    """
    def match(self, context, domain, *args):
        if "attrs" not in context.values or domain is None:
            return self.NO_MATCH

        attrs = context.values["attrs"]
        # match if all selected attributes in domain
        values = set(y for x in attrs for y in x)
        if len(values) > 0 and all(v in domain for v in values):
            return self.PERFECT_MATCH
        else:
            return self.NO_MATCH

class OWStationarity(OWWidget):
    name = "Stationarity Tests"
    description = "Augmented Dickey-Fuller and Kwiatkowski-Phillips-Schmidt-Shin Tests"
    icon = "icons/DataInfo.svg"
    priority = 120
    keywords = "time series, ADF, KPSS, stationarity"

    vars = settings.ContextSetting(None)
    settingsHandler = StationarityContextHandler()
    attrs: List[List[str]] = ContextSetting([])

    class Inputs:
        time_series = Input("Time series", Table)

    want_main_area = False
    buttons_area_orientation = None
    resizing_enabled = False

    class Error(OWWidget.Error):
        num_features = Msg("Data must contain at least {}.")
        no_class = Msg("Data must have a single target variable.")
        no_class_values = Msg("Target variable must have at least two values.")
        no_nonnan_data = Msg("No points with defined values.")
        same_variable = Msg("Select two different variables.")


    def __init__(self):
        super().__init__()
        self.data: Optional[Timeseries] = None
        self._vars_model = DomainModel(order=DomainModel.MIXED,
                                       valid_types=ContinuousVariable)
        self.vars = None
        self.data = None
        self.data_ADF = {}
        self.data_KPSS = {}
        self.options_box = gui.widgetBox(self.controlArea, "Variable")
        opts = dict(
            widget=self.options_box, master=self, orientation=Qt.Horizontal,
            callback=self.change_attributes)
        self.description1=gui.widgetLabel(gui.comboBox(value='vars', model=self._vars_model, **opts))
        label = "<center><b><font size = \"+2\" color = \"#8B0000\">Augmented Dickey-Fuller Test</font></b></center>"
        self.title1 = gui.widgetLabel(
                   gui.vBox(self.controlArea, box=''))
        self.title1.setText(label)
        self.description = gui.widgetLabel(
                   gui.vBox(self.controlArea, box=''))
        #       self.description = gui.widgetLabel(
 #           gui.vBox(self.controlArea, box="Augmented Dickey-Fuller Test"))
        label = "<center><br><b><font size = \"+2\" color = \"#8B0000\">  Kwiatkowski-Phillips-Schmidt-Shin Test </font></br></b></center>"
        self.title1 = gui.widgetLabel(
                   gui.vBox(self.controlArea, box=''))
        self.title1.setText(label)
        self.attributes = gui.widgetLabel(
            gui.vBox(self.controlArea, box=""))

    @Inputs.time_series
    def set_data(self, data: Table):
        self.closeContext()
        self.data = data and Timeseries.from_data_table(data)
        self.selected_data = None
        self.init_model()
        self.openContext(self.data.domain if self.data else None)
        self.apply_settings()
        self.vars=(self._vars_model[:1])[0]


        if data is None:
            self.data_ADF = self.data_KPSS = {}
            self.update_info()
        else:
            self.calculate()
            self.update_info()

    def calculate(self):
        vars_name = self.vars.name
        temp_data = (table_to_frame(self.data))[vars_name]
        self.data_ADF = self.adf_test(temp_data.dropna())
        self.data_KPSS = self.kpss_test(temp_data.dropna())

    def adf_test(self,data):
        tmp_result={}
        result = adfuller(data)
        labels = ['Test Statistic', 'p-value', '#Lags Used', '#Observations Used']
        for value, label in zip(result, labels):
            if (label=='p-value'):
                val=round(value,2)
            else:
                val=round(value,3)
            tmp_result[label]=str(val)
        for key in result[4]:
            key1 = "Critical Value ({0})".format(key)
            tmp_result[key1] = str(round(result[4][key],3))
        if ((result[0] < result[4]["5%"])&(result[1]<=0.05)):
            tmp_result['<font color=\'green\' > Reject Ho </font>']="<font color=\'green\' >Time Series is Stationary</font>"
        else:
            tmp_result['<font color=\'red\' > Failed to Reject Ho </font>']="<font color=\"red\" > Time Series is Non-Stationary</font>"
        return tmp_result

    def kpss_test(self,data):
        tmp_result={}
        result = kpss(data)
        labels = ['Test Statistic', 'p-value', '#Lags Used']
        for value, label in zip(result, labels):
            if (label=='p-value'):
                val=round(value,2)
            else:
                val=round(value,3)
            tmp_result[label]=str(round(value,3))
        for key in result[3]:
            key1 = "Critical Value ({0})".format(key)
            tmp_result[key1] = str(result[3][key])
        if ((result[0] < result[3]["5%"])&(result[1]<=0.05)):
            tmp_result['<font color=\'green\'> Reject Ho </font>']="<font color=\'green\' > Time Series is Stationary</font>"
        else:
            tmp_result['<font color=\'red\' > Failed to Reject Ho </font>']="<font color=\"red\" > Time Series is Non-Stationary</font>"
        return tmp_result


    def init_model(self):
        domain = None
        if self.data is not None:
            has_time_var = hasattr(self.data, "time_variable")

            def filter_vars(variables):
                return [var for var in variables if not has_time_var or
                        has_time_var and var != self.data.time_variable]

            domain = Domain(filter_vars(self.data.domain.attributes),
                            filter_vars(self.data.domain.class_vars),
                            filter_vars(self.data.domain.metas))
        self._vars_model.set_domain(domain)

    def apply_settings(self):
        for attrs, log, typ in zip(self.attrs):
            if typ == "spline":  # TODO - spline is missing
                typ = "line"
            self._add_editor({"vars": attrs, "is_log": log,
                              "type": CurveTypes.ITEMS.index(typ)})

    def update_info(self):
        style = """<style>
                       th { text-align: right; vertical-align: top; }
                       th, td { padding-top: 4px; line-height: 125%}
                    </style>"""

        def dict_as_table(d):
            return "<table>" + \
                   "".join(f"<tr><th>{label}: </th><td>" + \
                           '<br/>'.join(textwrap.wrap(value, width=60)) + \
                           "</td></tr>"
                           for label, value in d.items()) + \
                   "</table>"

        if not self.data_ADF:
            self.description.setText("No data.")
        else:
            self.description.setText(style + dict_as_table(self.data_ADF))
        self.attributes.setHidden(not self.data_KPSS)
        if self.data_KPSS:
            self.attributes.setText(
                style + dict_as_table(self.data_KPSS))

    def send_report(self):
        if self.data_ADF:
            self.report_items("Data table properties", self.data_ADF)
        if self.data_KPSS:
            self.report_items("Additional attributes", self.data_KPSS)

    def select_columns(self):
        self.Error.no_nonnan_data.clear()
        self.Error.same_variable.clear()
        self.selected_data = None
        if self.data is None:
            return

        domain = Domain(self.vars)
        data = self.data.transform(domain)
        valid_data = \
            np.flatnonzero(
                np.all(
                    np.isfinite(data.X),
                    axis=1)
            )

        if not valid_data.size:
            self.Error.no_nonnan_data()
            return

    def change_attributes(self):
        self.restart()

    def restart(self):
        self.selected_data=vars
        #self.select_columns()
        if self.selected_data is None:
            return
        self.calculate()
        self.update_info()


if __name__ == "__main__":  # pragma: no cover
    data = Table(TS.from_file('airpassengers'))
    #data=Orange.data.Table("iris")
    #domain = Domain(data.domain.attributes[:-1], data.domain.attributes[-1])
    #data = Timeseries.from_numpy(domain, data.X[:, :-1], data.X[:, -1])
    WidgetPreview(OWStationarity).run(set_data=data)



import Orange.data
import numpy as np
import pandas as pd
import seaborn as sns
from AnyQt.QtCore import Qt
from orangewidget import gui
from Orange.widgets import settings
from Orange.widgets.utils.itemmodels import DomainModel
from Orange.preprocess.preprocess import Normalize
from Orange.data.pandas_compat import  table_to_frame
from Orange.data import Table, Domain, ContinuousVariable
from orangecontrib.tools.widgets.utils import MatplotlibWidget

from orangecontrib.tools.widgets.mywidgets import OWWidget, Input, Msg

def safe_compare(group, value):
    """Manejo seguro de comparaciones con NumPy arrays"""
    if isinstance(value, np.ndarray):
        return np.array_equal(group, value)
    return group == value

class Pairsplot(OWWidget):
    # Nombre del widget como se verá en el lienzo
    name = 'Pairs plot'
    description = 'Pairs plot with some tunning features'
    icon = 'icons/Pairs.svg'
    keywords = ["widget", "tools"]
    category = 'Tools'

    class Inputs:
        data = Input("Data", Table)

    class Outputs:
        sample = None #Output("Sampled Data", Orange.data.Table)

    graph_name = "graph"  # QGraphicsView (pg.PlotWidget)
    settingsHandler = settings.DomainContextHandler(
        match_values=settings.DomainContextHandler.MATCH_VALUES_CLASS)
    attr_numvars=settings.ContextSetting(None)
    attr_x = settings.ContextSetting(None)
    attr_y = settings.ContextSetting(None)
    attr_size = settings.Setting(1)
    key_size = settings.Setting(1)
    attr_col = settings.ContextSetting('b')
    key_col = settings.ContextSetting('Blue')
    attr_sym = settings.Setting('o')
    key_sym = settings.Setting('Circle')
    attr_bins = settings.Setting(10)
    key_bins = settings.Setting(10)
    axes = None

    default_background_color = np.array([0, 0xbf, 0xff])

    SYMBOL_TYPES = (
        ('Circle', 'o'),
        ('Square', 's'),
        ('Triangle', 'v'),
        ('Diamond', 'D'),
        ('Plus', 'P'),
        ('Cross', 'X'),
    )
    COLOR_NAMES = (
        ('Blue', 'b'),
        ('Red', 'r'),
        ('Green', 'g'),
        ('Cyan', 'c'),
        ('Yellow', 'y'),
        ('Magenta', 'm'),
        ('Black','k')
    )
    def resizeEvent(self,e):
        #if not self._flag:
        #    self._flag = True
        #    self.scale_image()
        #    QtCore.QTimer.singleShot(50, lambda: setattr(self,"_flag",False))
        #super().resizeEvent(e)
        self.restart()


    class Error(OWWidget.Error):
        num_features = Msg("Data must contain at least {}.")
        no_class = Msg("Data must have a single target variable.")
        no_class_values = Msg("Target variable must have at least two values.")
        no_nonnan_data = Msg("No points with defined values.")
        same_variable = Msg("Select two different variables.")


    def __init__(self):
        super().__init__()
        self.Error.add_message("same_variable")
        self.data = None
        self.selected_data = None
        self.min_x = self.max_x = self.min_y = self.max_y = None
        self._add_graph()
        self.var_model = DomainModel(valid_types=ContinuousVariable,
                                     order=DomainModel.ATTRIBUTES)
        self.attr_numvars=len(self.var_model)
        self.parameters_box = gui.widgetBox(self.controlArea, "Parameters")
        opts1 = dict(
            widget=self.parameters_box, master=self, orientation=Qt.Horizontal,
            callback=self.change_parameters, sendSelectedValue=True, contentsLength=14)
        gui.comboBox(value='key_sym', label='Plot Symbol',
                     items=[i[0] for i in self.SYMBOL_TYPES], **opts1)
        gui.comboBox(value='key_col', label='Color',
                     items=[i[0] for i in self.COLOR_NAMES], **opts1)
        gui.spin(widget=self.parameters_box, master=self, value='attr_size', minv=1, maxv=30, step=1, label="Symbol Size:",
                 alignment=Qt.AlignRight, controlWidth=80, callback=self.change_parameters)
        gui.spin(widget=self.parameters_box, master=self, value='attr_bins', minv=2, maxv=500, step=1, label="Histogram Bins:",
                 alignment=Qt.AlignRight, controlWidth=80, callback=self.change_parameters)

        gui.rubber(self.controlArea)


    def change_parameters(self):
        self.attr_sym = dict(self.SYMBOL_TYPES)[self.key_sym]
        self.attr_col = dict(self.COLOR_NAMES)[self.key_col]
        self.restart()

    def _add_graph(self):
        self.graph = MatplotlibWidget()
        self.mainArea.layout().addWidget(self.graph)

    def change_attributes(self):
        self.restart()

    def restart(self):
        self.clear_plot()
        self.select_columns_optimized()
        if self.selected_data is None:
            return
        self.replot()

    #def keyPressEvent(self, e):
    #    """Bind 'back' key to step back"""
    #    if (e.modifiers(), e.key()) in self.key_actions:
    #        fun = self.key_actions[(e.modifiers(), e.key())]
    #        fun(self)
    #    else:
    #        super().keyPressEvent(e)

    #def resizeEvent(self, event):
    #    self.restart()

    ##############################
    # Signals and data-handling

    @Inputs.data
    def set_data(self, data):

        self.closeContext()
        self.Error.clear()

        self.selected_data = None
        self.data = None
        self.clear_plot()

        if data:
            self.var_model.set_domain(data.domain)
            if len(self.var_model) < 1:
                self.Error.num_features("Only one numeric variable")
                data = None
        if not data:
            self.var_model.set_domain(None)
            return

        self.data = data
        self.attr_x = self.var_model[0]
        self.attr_y = self.var_model[1]
        self.openContext(self.data)
        self.attr_numvars=len(self.var_model)
        self.restart()


    def select_columns_optimized(self):
        if "no_nonnan_data" not in self.Error.__dict__:
            self.Error.add_message("no_nonnan_data", "No valid non-NaN data available")
        self.Error.no_nonnan_data.clear()
        if hasattr(self.Error, "same_variable"):
            self.Error.same_variable(shown=False)

        if self.data is None or self.attr_x is self.attr_y:
            if self.attr_x is self.attr_y:
                self.Error.same_variable()
            return

        domain = Domain([self.attr_x, self.attr_y])
        data = self.data.transform(domain)

        valid_data = np.all(np.isfinite(data.X), axis=1)
        if not np.any(valid_data):
            self.Error.no_nonnan_data()
            return

        self.selected_data = data[valid_data]

    def select_columns(self):
        if "no_nonnan_data" not in self.Error.__dict__:
            self.Error.add_message("no_nonnan_data", "No valid non-NaN data available")
        self.Error.no_nonnan_data.clear()
        if hasattr(self.Error, "same_variable"):
            self.Error.same_variable(shown=False)  # En lugar de borrar, oculta el mensaje
        self.selected_data = None
        if self.data is None:
            return

        if self.attr_x is self.attr_y:
            self.Error.same_variable()
            return
        domain = Domain([self.attr_x, self.attr_y])
        data = self.data.transform(domain)
        valid_data = \
            np.flatnonzero(
                np.all(
                    np.isfinite(data.X),
                    axis=1)
            )

        if not valid_data.size:
            self.Error.no_nonnan_data()
            return

        data = data[valid_data]
        self.selected_data=data



    def send_report(self):
        if self.data is None:
            return
        self.report_plot()

    ##############################
    # Plot-related methods

    def clear_plot(self):
        self.graph.getFigure().clf()


    def replot(self):
        if self.data is None or self.selected_data is None:
            self.clear_plot()
            return
        self.plot()

    def plot(self):
        sns.set()
        self.graph.getFigure().clf()
        data = table_to_frame(self.data, include_metas=True)

        # A layout of nxn subplots where n= self.attr_numvars. For efficiency, maximum n=5, discard the others.
        n = self.attr_numvars
        if n>5:
            n=5
        #fig, axes = plt.subplots(4, 4, figsize=(12, 8), sharex="col", tight_layout=True)
        self.graph.axes = self.graph.getFigure().subplots(n, n, sharex="col")

        for i in range(n):
            for j in range(n):
                # If this is the lower-triangule, add a scatterlpot for each group.
                if i > j:
                    x=data[self.data.domain.attributes[i].name]
                    y=data[self.data.domain.attributes[j].name]
                    self.graph.axes[i,j].plot(x,y, marker=self.attr_sym, color=self.attr_col,markersize=self.attr_size,linestyle='None')
                else:
                    if i < j:
                        # Remove axis and grid
                        self.graph.axes[i, j].grid(False)
                        self.graph.axes[i, j].axis('off')
                        # Add correlation coefficient
                        x = data[self.data.domain.attributes[i].name]
                        y = data[self.data.domain.attributes[j].name]
                        correlation = np.corrcoef(x, y)[0, 1]
                        cor = 'Correlation coefficient:\n {:.4f}'.format(correlation)
                        left, width = .25, .5
                        bottom, height = .25, .5
                        right = left + width
                        top = bottom + height
                        self.graph.axes[i, j].text(0.5 * (left + right), 0.5 * (bottom + top), cor,
                                                   horizontalalignment='center',
                                                   verticalalignment='center',
                                                   transform=self.graph.axes[i, j].transAxes)
                    else:
                       # If this is the main diagonal, add histograms
                         x=data[self.data.domain.attributes[i].name]
                         self.graph.axes[i,j].hist(x,color=self.attr_col, bins=self.attr_bins, alpha=0.5)
                if j == 0:
                    self.graph.axes[i,j].set_ylabel(self.data.domain.attributes[i].name)
                if i == (n-1):
                    self.graph.axes[i,j].set_xlabel(self.data.domain.attributes[j].name)



#        self.graph.getFigure().tight_layout()
        self.graph.draw()

    def _format_label(self, x, y, step):
        return \
            f"<b>Step {step}:</b><br/>" \
            f"{x:.3f}, {y:.3f}<br/>" \
            f"Cost: {self.learner.j(np.array([x, y])):.5f}"


if __name__ == '__main__':
    from Orange.widgets.utils.widgetpreview import WidgetPreview  # since Orange 3.20.0
    file = pd.read_csv('../datasets/auto-mpg.csv')
    #WidgetPreview(Pairsplot).run(Orange.data.Table(file))

    WidgetPreview(Pairsplot).run(set_data=Orange.data.Table("iris"))



from os.path import join, dirname
from Orange.data.table import dataset_dirs
from Orange.data import Table


dataset_dirs.insert(0, join(dirname(__file__), "datasets"))
del join, dirname, dataset_dirs


# Remove this when we require Orange 3.34
if not hasattr(Table, "get_column"):
    import scipy.sparse as sp
    import numpy as np

    def get_column(self, column):
        col, _ = self.get_column_view(column)
        if sp.issparse(col):
            col = col.toarray().reshape(-1)
        if self.domain[column].is_primitive():
            col = col.astype(np.float64)
        return col

    Table.get_column = get_column



"""Mixin class for errors, warnings and information

A class derived from `OWBaseWidget` can include member classes `Error`, `Warning`
and `Information`, derived from the same-named `OWBaseWidget` classes. Each of
those contains members that are instances of `UnboundMsg`, which is, for
convenience, also exposed as `Orange.widgets.widget.Msg`. These members
represent all possible errors, like `Error.no_discrete_vars`, with exception
of the deprecated old-style errors.

When the widget is instantiated, classes `Error`, `Warning` and `Information`
are instantiated and bound to the widget: their attribute `widget` is the link
to the widget that instantiated them. Their member messages are replaced with
instances of `_BoundMsg`, which are bound to the group through the `group`
attribute.

A message is shown by calling, e.g. `self.Error.no_discrete_vars()`. The call
formats the message and tells the group to activate it::

    self.formatted = self.format(*args, **kwargs)
    self.group.activate_msg(self)

The group adds it to the dictionary of active messages (attribute `active`)
and emits the signal `messageActivated`. The signal is connected to the
widget's method `update_widget_state`, which shows the message in the bar, and
`WidgetManager`'s `__on_widget_state_changed`, which manages the icons on the
canvas.

Clearing messages work analogously.
"""
import sys
import numpy as np
import traceback
from operator import attrgetter
from warnings import warn
from inspect import getattr_static
from typing import Optional

from AnyQt.QtWidgets import QStyle, QSizePolicy

from orangewidget import gui
from orangewidget.utils.messagewidget import MessagesWidget


class UnboundMsg(str):
    """
    The class used for declaring messages in classes derived from
    MessageGroup. When instantiating the message group, instances of this
    class are replaced by instances of `_BoundMsg` that are bound to the group.

    Note: this class is aliased to `Orange.widgets.widget.Msg`.
    """
    def __new__(cls, msg):
        return str.__new__(cls, msg)

    def bind(self, group, owner_class=None):
        return _BoundMsg(self, group, owner_class)

    # The method is implemented in _BoundMsg
    # pylint: disable=unused-variable
    def __call__(self, *args, shown=True, exc_info=None, **kwargs):
        """
        Show the message, or hide it if `show` is set `False`
        `*args` and `**kwargs` are passed to the `format` method.

        Args:
            shown (bool): keyword-only argument that can be set to `False` to
                hide the message
            exc_info (Union[BaseException, bool, None]): Optional exception
                instance whose traceback to store in the message. Can also be
                a `True` value in which case the exception is retrieved from
                sys.exc_info()
            *args: arguments for `format`
            **kwargs: keyword arguments for `format`
        """
        raise RuntimeError("Message is not bound")

    # The method is implemented in _BoundMsg
    def clear(self):
        """Remove the message."""
        raise RuntimeError("Message is not bound")

    # The method is implemented in _BoundMsg
    def is_shown(self):
        """Return True if message is currently displayed."""
        raise RuntimeError("Message is not bound")

    # Ensure that two instance of message are always different
    # In particular, there may be multiple messages "{}".
    def __hash__(self):
        return id(self)

    def __eq__(self, other):
        return self is other


class _BoundMsg(UnboundMsg):
    """
    A message that is bound to the group.

    Instances of this class provide the call operator for raising the message,
    and method `clear` for removing it.

    When the message is called, the arguments are passed to the message's
    format` method and the resulting string is stored in the attribute
    `formatted`.

    Attributes:
        group (MessageGroup): the group to which this message belongs
        owner_class (OWBaseWidget): the class in which the message is defined
        formatted (str): formatted message

    """
    def __new__(cls, unbound_msg, group, owner_class=None):
        self = UnboundMsg.__new__(cls, unbound_msg)
        self.group = group
        self.owner_class = owner_class
        self.formatted = ""
        self.tb = None  # type: Optional[str]
        return self

    def __call__(self, *args, shown=True, exc_info=None, **kwargs):
        self.tb = None
        if not shown:
            self.clear()
        else:
            self.formatted = self.format(*args, **kwargs)
            if exc_info:
                if isinstance(exc_info, BaseException):
                    exc_info = (type(exc_info), exc_info,
                                exc_info.__traceback__)
                elif not isinstance(exc_info, tuple):
                    exc_info = sys.exc_info()
                if exc_info is not None:
                    self.tb = "".join(traceback.format_exception(*exc_info))
            self.group.activate_msg(self)

    def clear(self):
        self.group.deactivate_msg(self)

    def is_shown(self):
        return self in self.group.active

    def __str__(self):
        return self.formatted


class _OldStyleMsg(_BoundMsg):
    """
    Class for handling the old-style messages.

    Instances of this class are instantiated by the old methods `error`,
    `warning` and `information` and added to the list of active messages,
    with their old-style id's as keys. Instance of `_OldStyleMsg` are not
    members of message groups.
    """
    def __new__(cls, text, group):
        self = _BoundMsg.__new__(cls, text, group)
        self.formatted = text
        return self


class MessageGroup:
    """
    A groups of messages, e.g. errors, warnings, information messages

    Widget's `__init__` searches for instances of this class among the widget's
    class member and instantiates them.

    Attributes:
        widget (widget.OWBaseWidget): the widget instance to which the group belongs
    """
    def __init__(self, widget):
        self.widget = widget
        # Note: active messages are stored in the dictionary, in which
        # the key and the corresponding value are one and the same object,
        # except for old-style classes, for which the key is an (old-style)
        # id. When we remove support for old-style messages (Orange 4),
        # this dictionary  can be replaced with a set.
        self._active = {}
        self._general = UnboundMsg("{}")
        self._bind_messages()

    @property
    def active(self):
        """
        Sequence[_BoundMsg]: Sequence of all currently active messages.
        """
        return self._active.values()

    def _bind_messages(self):
        def bind_subgroup(subgroup, widget_class):
            for name, msg in subgroup.__dict__.items():
                if type(msg) is UnboundMsg:
                    msg = msg.bind(self, widget_class)
                    self.__dict__[name] = msg

        for group in type(self).mro():
            for widget_class in type(self.widget).mro():
                a=1
                if any(np.array_equal(group, value) if isinstance(value, np.ndarray) else group == value for value in widget_class.__dict__.values()):
                #if group in widget_class.__dict__.values():
                    break
            else:
                # MessageGroups outside widget classes (e.g. mixins)
                widget_class = None
            bind_subgroup(group, widget_class)

        # This binds `_general` -- and any similar cases in which a message is
        # added to the instance and not as class attribute
        bind_subgroup(self, None)

    def add_message(self, name, msg="{}"):
        """Add and bind message to a group that is already instantiated
        and bound.

        If the message with that name already exists, the method does nothing.
        The method is used by helpers like this (simplified) one::

            def check_results(results, msg_group):
                msg_group.add_message("invalid_results",
                                      "Results do not include any data")
                msg_group.invalid_results.clear()
                if results.data is None:
                    msg_group.invalid_results()

        The helper is called from several widgets with
        `check_results(results, self.Error)`

        Args:
            name (str): the name of the member with the message
            msg (str or UnboundMsg): message text or instance (default `"{}"`)
        """
        if not isinstance(msg, UnboundMsg):
            msg = UnboundMsg(msg)
        if name not in self.__dict__:
            self.__dict__[name] = msg.bind(self)

    def activate_msg(self, msg, msg_id=None):
        """Activate a message and emit the signal messageActivated

        Args:
            msg (_BoundMsg): the message to activate
            msg_id (int): id for old-style message (to be removed in the future)
        """
        key = msg if msg_id is None else msg_id
        if self._active.get(key) == msg:
            self.widget.messageActivated.emit(msg)
            return
        self._active[key] = msg
        self.widget.messageActivated.emit(msg)

    def deactivate_msg(self, msg):
        """Deactivate a message and emit the signal messageDeactivated.

        Args:
            msg (_BoundMsg): the message to deactivate
        """
        if msg not in self._active:
            return
        inst_msg = self._active.pop(msg)
        self.widget.messageDeactivated.emit(
            inst_msg if isinstance(msg, int) else msg)

        # When when we no longer support old-style messages, replace with:
        # if msg not in self._active:
        #     return
        # del self._active[msg]
        # self.widget.messageDeactivated.emit(msg)

    # self has default value to avoid PyCharm warnings when calling
    # self.Error.clear(): PyCharm doesn't know that Error is instantiated
    def clear(self=None, *, owner=None):
        """Deactivate all active message from this group."""
        for msg in list(self._active):
            if owner is None or msg.owner_class is owner:
                self.deactivate_msg(msg)

    def _add_general(self, id_or_text, text, shown):
        """Handler for methods `error`, `warning` and `information`;
        do not call directly.

        The message is shown as general message. This method also
        handles deprecated messages with id's."""
        if id_or_text is None or id_or_text == "":
            self._general.clear()
        elif isinstance(id_or_text, str):
            self._general(id_or_text, shown=shown)
        # remaining cases handle deprecated messages with id's
        elif text:
            self.activate_msg(_OldStyleMsg(text, self), id_or_text)
        elif isinstance(id_or_text, list):
            for msg_id in id_or_text:
                self.deactivate_msg(msg_id)
        else:
            self.deactivate_msg(id_or_text)


class MessagesMixin:
    """
    Base class for message mixins. The class provides a constructor for
    instantiating and binding message groups.

    Widgets should use `WidgetMessageMixin rather than this class.
    """
    def __init__(self):
        # type(self).__dict__ wouldn't return inherited messages, hence dir
        self.message_groups = []
        for name in dir(self):
            group_class = getattr_static(self, name)
            if isinstance(group_class, type) and \
                    issubclass(group_class, MessageGroup) and \
                    group_class is not MessageGroup:
                bound_group = group_class(self)
                setattr(self, name, bound_group)
                self.message_groups.append(bound_group)
        self.message_groups.sort(key=attrgetter("severity"), reverse=True)


class WidgetMessagesMixin(MessagesMixin):
    """
    Provide the necessary methods for handling messages in widgets.

    The class defines member classes `Error`, `Warning` and `Information` that
    serve as base classes for these message groups.
    """
    class Error(MessageGroup):
        """Base class for groups of error messages in widgets"""
        severity = 3
        icon_path = gui.resource_filename("icons/error.png")
        bar_background = "#ffc6c6"
        bar_icon = QStyle.SP_MessageBoxCritical

    class Warning(MessageGroup):
        """Base class for groups of warning messages in widgets"""
        severity = 2
        icon_path = gui.resource_filename("icons/warning.png")
        bar_background = "#ffffc9"
        bar_icon = QStyle.SP_MessageBoxWarning

    class Information(MessageGroup):
        """Base class for groups of information messages in widgets"""
        severity = 1
        icon_path = gui.resource_filename("icons/information.png")
        bar_background = "#ceceff"
        bar_icon = QStyle.SP_MessageBoxInformation

    def __init__(self):
        super().__init__()
        self.message_bar = None
        self.messageActivated.connect(self.update_message_state)
        self.messageDeactivated.connect(self.update_message_state)

    def clear_messages(self):
        """Clear all messages"""
        for group in self.message_groups:
            group.clear()

    def update_message_state(self):
        """Show and update (or hide) the content of the widget's message bar.

        The method is connected to widget's signals `messageActivated` and
        `messageDeactivated`.
        """
        if self.message_bar is None:
            return
        assert isinstance(self.message_bar, MessagesWidget)

        def msg(m):
            # type: (_BoundMsg) -> MessagesWidget.Message
            text = str(m)
            extra = ""
            if "\n" in text:
                text, extra = text.split("\n", 1)

            return MessagesWidget.Message(
                MessagesWidget.Severity(m.group.severity),
                text=text, informativeText=extra,
                detailedText=m.tb if m.tb else ""
            )

        messages = [msg
                    for group in self.message_groups
                    for msg in group.active]

        self.message_bar.clear()
        if messages:
            self.message_bar.setMessages((m, msg(m)) for m in messages)
        self.message_bar.setVisible(bool(messages))

    def insert_message_bar(self):
        """Insert message bar into the widget.

        This method must be called at the appropriate place in the widget
        layout setup by any widget that is using this mixin."""
        self.message_bar = MessagesWidget(self)
        self.message_bar.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Maximum)
        self.layout().addWidget(self.message_bar)
        self.message_bar.setVisible(False)

    # pylint doesn't know that Information, Error and Warning are instantiated
    # and thus the methods are bound
    # pylint: disable=no-value-for-parameter
    # This class and classes Information, Error and Warning are friends
    # pylint: disable=protected-access
    @staticmethod
    def _warn_obsolete(text_or_id, what):
        if not isinstance(text_or_id, str) and text_or_id is not None:
            warn("'{}' with id is deprecated; use {} class".
                 format(what, what.title()), stacklevel=3)

    def information(self, text_or_id=None, text=None, shown=True):
        self._warn_obsolete(text_or_id, "information")
        self.Information._add_general(text_or_id, text, shown)

    def warning(self, text_or_id=None, text=None, shown=True):
        self._warn_obsolete(text_or_id, "warning")
        self.Warning._add_general(text_or_id, text, shown)

    def error(self, text_or_id=None, text=None, shown=True):
        self._warn_obsolete(text_or_id, "error")
        self.Error._add_general(text_or_id, text, shown)



from os import path, walk
from setuptools import setup, find_packages
from setuptools.dist import Distribution

NAME = "Orange3-Tools"

VERSION = "0.0.12"

DESCRIPTION = "Tools para Asignatura Big Data Master EERR"
LONG_DESCRIPTION = ""

LICENSE = "GPL-3.0"

KEYWORDS = (
    # [PyPi](https://pypi.python.org) packages with keyword "orange3 add-on"
    # can be installed using the Orange Add-on Manager
    'orange3 add-on',
)

PACKAGES = find_packages()

print(PACKAGES)

PACKAGE_DATA = {
    'orangecontrib.tools.widgets': ['icons/*'],
}

DATA_FILES = [
    # Data files that will be installed outside site-packages folder
]

INSTALL_REQUIRES = [
    'anyqt',
    'pandas',
    'BeautifulSoup4',
    'numpy',
    'seaborn',
    'Orange3 >=3.31.1',
    'orange-widget-base',
    'scikit-learn',
    'pyqtgraph',
    'mysql-connector-python',
]

EXTRAS_REQUIRE = {
    'doc': ['sphinx', 'recommonmark', 'sphinx_rtd_theme'],
    'test': ['coverage'],
}

ENTRY_POINTS = {
    # Entry points that marks this package as an orange add-on. If set, addon will
    # be shown in the add-ons manager even if not published on PyPi.
    'orange3.addon': (
        'tools = orangecontrib.tools',
    ),
    # Entry point used to specify packages containing tutorials accessible
    # from welcome screen. Tutorials are saved Orange Workflows (.ows files).
    #'orange.widgets.tutorials': (
        # Syntax: any_text = path.to.package.containing.tutorials
    #    'educationaltutorials = orangecontrib.educational.tutorials',
    #),

    # Entry point used to specify packages containing widgets.
    'orange.widgets': (
        # Syntax: category name = path.to.package.containing.widgets
        # Widget category specification can be seen in
        #    orangecontrib/example/widgets/__init__.py
        'Tools = orangecontrib.tools.widgets',
    ),

    # Register widget help
    #"orange.canvas.help": (
    #    'html-index = orangecontrib.educational.widgets:WIDGET_HELP_PATH',)
}

NAMESPACE_PACKAGES = ["orangecontrib"]

AUTHOR = 'Julio J. Melero'
AUTHOR_EMAIL = 'melero@unizar.es'
URL = ""
DOWNLOAD_URL = ""

#def include_documentation(local_dir, install_dir):
#    global DATA_FILES

#    doc_files = []
#    for dirpath, _, files in walk(local_dir):
#        doc_files.append((dirpath.replace(local_dir, install_dir),
#                          [path.join(dirpath, f) for f in files]))
#    DATA_FILES.extend(doc_files)

class BinaryDistribution(Distribution):
    """Distribution which always forces a binary package with platform name"""
    def has_ext_modules(foo):
        return True

if __name__ == '__main__':
    #include_documentation('doc/_build/html', 'help/orange3-educational')
    setup(
        name=NAME,
        version=VERSION,
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        long_description_content_type='text/markdown',
        license=LICENSE,
        packages=PACKAGES,
        package_data=PACKAGE_DATA,
        data_files=DATA_FILES,
        install_requires=INSTALL_REQUIRES,
        extras_require=EXTRAS_REQUIRE,
        entry_points=ENTRY_POINTS,
        keywords=KEYWORDS,
        #namespace_packages=NAMESPACE_PACKAGES,
        include_package_data=True,
        zip_safe=False,
        author=AUTHOR,
        author_email=AUTHOR_EMAIL,
        url=URL,
        download_url=DOWNLOAD_URL,
        classifiers = []
#        distclass=BinaryDistribution
    )




from orangewidget.utils.widgetpreview import WidgetPreview

from Orange.data import Domain

from orangecontrib.timeseries import Timeseries, ARIMA
from orangecontrib.timeseries.widgets._owmodel import OWBaseModel
from AnyQt.QtCore import QTimer, Qt
from AnyQt.QtWidgets import QFormLayout

from Orange.data import Table
from Orange.widgets import widget, gui, settings
from Orange.widgets.widget import Input, Output
from orangecontrib.timeseries import Timeseries
from orangecontrib.timeseries.models import _BaseModel
import statsmodels.api as sm
import numpy as np
class MY_ARIMA(_BaseModel):
    """Autoregressive integrated moving average (ARIMA) model

    An auto regression (AR) and moving average (MA) model with differencing.

    If exogenous variables are provided in fit() method, this becomes an
    ARIMAX model.

    Parameters
    ----------
    order : tuple (p, d, q)
        Tuple of three non-negative integers: (p) the AR order, (d) the
        degree of differencing, and (q) the order of MA model.
        If d = 0, this becomes an ARMA model.

    Returns
    -------
    unfitted_model
    """
    REQUIRES_STATIONARY = False
#    __wrapped__ = statsmodels.tsa.arima.model.ARIMA
    __wrapped__ = sm.tsa.arima.ARIMA

    def __init__(self, order=(1, 0, 0),seasonal_order=(0,0,0,0), use_exog=False):
        super().__init__()
        self.order = order
        self.seasonal_order = seasonal_order
        self.use_exog = use_exog
        self._model_kwargs.update(order=order,seasonal_order = seasonal_order)

    def __str__(self):
        return '{}({})({})'.format('SAR{}MA{}'.format('I' if self.order[1] else '',
                                                 'X' if self.use_exog else ''),
                               ','.join(map(str, self.order)),','.join(map(str,self.seasonal_order)))

    def _predict(self, steps, exog, alpha):
        pred_res = self.results.get_forecast(steps, exog=exog)
        forecast = pred_res.predicted_mean
        confint = pred_res.conf_int(alpha=alpha)
        return np.c_[forecast, confint].T

    def _before_init(self, endog, exog):
        exog = exog if self.use_exog else None
        if len(endog) == 0:
            raise ValueError('Need an endogenous (target) variable to fit')
        return endog, exog

    def _fittedvalues(self):
        # Statsmodels supports different args whether series is
        # differentiated (order has d) or not. -- stupid statsmodels
        kwargs = dict(typ='levels') if self.order[1] > 0 else {}
        return self.results.predict(**kwargs)



class OWSARIMAModel(OWBaseModel):
    name = 'SARIMA Model'
    description = 'Model the time series using ARMA, ARIMA, or SARIMA.'
    icon = 'icons/ARIMA.svg'
    priority = 210

    p = settings.Setting(1)
    d = settings.Setting(0)
    q = settings.Setting(0)
    s_p = settings.Setting(0)
    s_d = settings.Setting(0)
    s_q = settings.Setting(0)
    s_s = settings.Setting(0)

    class Inputs(OWBaseModel.Inputs):
        exogenous_data = Input("Exogenous data", Timeseries)

    def __init__(self):
        super().__init__()
        self.exog_data = None

    @Inputs.exogenous_data
    def set_exog_data(self, data):
        self.exog_data = data
        self.update_model()

    def add_main_layout(self):
        layout = QFormLayout()
        self.controlArea.layout().addLayout(layout)
        kwargs = dict(controlWidth=50, alignment=Qt.AlignRight,
                      callback=self.apply.deferred)
        layout.addRow('Auto-regression order (p):',
                      gui.spin(None, self, 'p', 0, 100, **kwargs))
        layout.addRow('Differencing degree (d):',
                      gui.spin(None, self, 'd', 0, 2, **kwargs))
        layout.addRow('Moving average order (q):',
                      gui.spin(None, self, 'q', 0, 100, **kwargs))
        layout.addRow('Seasonal Auto-regression order (P):',
                      gui.spin(None, self, 's_p', 0, 100, **kwargs))
        layout.addRow('Seasonal Differencing degree (D):',
                      gui.spin(None, self, 's_d', 0, 2, **kwargs))
        layout.addRow('Seasonal Moving average order (Q):',
                      gui.spin(None, self, 's_q', 0, 100, **kwargs))
        layout.addRow('Seasonality order (s):',
                      gui.spin(None, self, 's_s', 0, 100, **kwargs))


    def forecast(self, model):
        return model.predict(self.forecast_steps,
                             exog=self.exog_data,
                             alpha=1 - self.forecast_confint / 100,
                             as_table=True)

    def create_learner(self):
        return MY_ARIMA((self.p, self.d, self.q),(self.s_p,self.s_d,self.s_q,self.s_s), self.exog_data is not None)


if __name__ == "__main__":
    data = Timeseries.from_file('airpassengers')
    domain = Domain(data.domain.attributes[:-1], data.domain.attributes[-1])
    data = Timeseries.from_numpy(domain, data.X[:, :-1], data.X[:, -1])
    WidgetPreview(OWSARIMAModel).run(set_data=data)



import Orange.data
import numpy as np
import seaborn as sns
from AnyQt.QtCore import Qt
from orangewidget import gui
from Orange.widgets.widget import OWWidget, Msg, Input
from Orange.widgets import settings
from Orange.widgets.utils.itemmodels import DomainModel
from Orange.preprocess.preprocess import Normalize
from Orange.data.pandas_compat import  table_to_frame
from Orange.data import Table, Domain, ContinuousVariable
from orangecontrib.tools.widgets.utils import MatplotlibWidget

class Scaterplot(OWWidget):
    # Nombre del widget como se verá en el lienzo
    name = 'Scatter plot'
    description = 'Scatter plot with some tunning features'
    icon = 'icons/sscater.svg'
    keywords = ["widget", "tools"]
    category = 'Tools'

    class Inputs:
        data = Input("Data", Table)

    class Outputs:
        sample = None #Output("Sampled Data", Orange.data.Table)

    graph_name = "graph"  # QGraphicsView (pg.PlotWidget)
    settingsHandler = settings.DomainContextHandler(
        match_values=settings.DomainContextHandler.MATCH_VALUES_CLASS)
    attr_x = settings.ContextSetting(None)
    attr_y = settings.ContextSetting(None)
    attr_size = settings.Setting(5)
    key_size = settings.Setting(5)
    attr_col = settings.ContextSetting('b')
    key_col = settings.ContextSetting('Blue')
    attr_sym = settings.Setting('o')
    key_sym = settings.Setting('Circle')
    axes = None

    default_background_color = [0, 0xbf, 0xff]

    SYMBOL_TYPES = (
        ('Circle', 'o'),
        ('Square', 's'),
        ('Triangle', 'v'),
        ('Diamond', 'D'),
        ('Plus', 'P'),
        ('Cross', 'X'),
    )
    COLOR_NAMES = (
        ('Blue', 'b'),
        ('Red', 'r'),
        ('Green', 'g'),
        ('Cyan', 'c'),
        ('Yellow', 'y'),
        ('Magenta', 'm'),
        ('Black','k')
    )

    class Error(OWWidget.Error):
        num_features = Msg("Data must contain at least {}.")
        no_class = Msg("Data must have a single target variable.")
        no_class_values = Msg("Target variable must have at least two values.")
        no_nonnan_data = Msg("No points with defined values.")
        same_variable = Msg("Select two different variables.")

    def __init__(self):
        super().__init__()
        self.data = None
        self.selected_data = None
        self.min_x = self.max_x = self.min_y = self.max_y = None
        self._add_graph()
        self.var_model = DomainModel(valid_types=ContinuousVariable,
                                     order=DomainModel.ATTRIBUTES)
        self.options_box = gui.widgetBox(self.controlArea, "Variables")
        opts = dict(
            widget=self.options_box, master=self, orientation=Qt.Horizontal,
            callback=self.change_attributes)
        gui.comboBox(value='attr_x', model=self.var_model, **opts)
        gui.comboBox(value='attr_y', model=self.var_model, **opts)
        self.parameters_box = gui.widgetBox(self.controlArea, "Parameters")
        opts1 = dict(
            widget=self.parameters_box, master=self, orientation=Qt.Horizontal,
            callback=self.change_parameters, sendSelectedValue=True, contentsLength=14)
        gui.comboBox(value='key_sym', label='Symbol',
                     items=[i[0] for i in self.SYMBOL_TYPES], **opts1)
        gui.comboBox(value='key_col', label='Color',
                     items=[i[0] for i in self.COLOR_NAMES], **opts1)
        gui.spin(widget=self.parameters_box, master=self, value='attr_size', minv=1, maxv=30, step=1, label="Size:",
                 alignment=Qt.AlignRight, controlWidth=80, callback=self.change_parameters)
        gui.rubber(self.controlArea)

    def change_parameters(self):
        self.attr_sym = dict(self.SYMBOL_TYPES)[self.key_sym]
        self.attr_col = dict(self.COLOR_NAMES)[self.key_col]
        self.restart()

    def _add_graph(self):
        self.graph = MatplotlibWidget()
        self.mainArea.layout().addWidget(self.graph)

    def change_attributes(self):
        self.restart()

    def restart(self):
        self.clear_plot()
        self.select_columns()
        if self.selected_data is None:
            return
        self.replot()

    #def keyPressEvent(self, e):
    #    """Bind 'back' key to step back"""
    #    if (e.modifiers(), e.key()) in self.key_actions:
    #        fun = self.key_actions[(e.modifiers(), e.key())]
    #        fun(self)
    #    else:
    #        super().keyPressEvent(e)

    def resizeEvent(self, event):
        self.restart()

    ##############################
    # Signals and data-handling

    @Inputs.data
    def set_data(self, data):

        self.closeContext()
        self.Error.clear()

        self.selected_data = None
        self.data = None
        self.clear_plot()

        if data:
            self.var_model.set_domain(data.domain)
            if len(self.var_model) < 1:
                self.Error.num_features("Only one numeric variable")
                data = None
        if not data:
            self.var_model.set_domain(None)
            return

        self.data = data
        self.attr_x = self.var_model[0]
        self.attr_y = self.var_model[1]
        self.openContext(self.data)
        self.restart()


    def select_columns(self):
        self.Error.no_nonnan_data.clear()
        self.Error.same_variable.clear()
        self.selected_data = None
        if self.data is None:
            return

        if self.attr_x is self.attr_y:
            self.Error.same_variable()
            return
        domain = Domain([self.attr_x, self.attr_y])
        data = self.data.transform(domain)
        valid_data = \
            np.flatnonzero(
                np.all(
                    np.isfinite(data.X),
                    axis=1)
            )

        if not valid_data.size:
            self.Error.no_nonnan_data()
            return

        data = data[valid_data]
        self.selected_data=data


    def send_report(self):
        if self.data is None:
            return
        self.report_plot()

    ##############################
    # Plot-related methods

    def clear_plot(self):
        self.graph.getFigure().clf()

    def replot(self):
        if self.data is None or self.selected_data is None:
            self.clear_plot()
            return
        self.plot()

    def plot(self):
        self.graph.getFigure().clf()
        data = table_to_frame(self.data, include_metas=True)
        x = self.selected_data.domain[0].name
        y = self.selected_data.domain[1].name
        sns.set()
        self.axes = self.graph.getFigure().add_subplot(111)
        self.axes.plot(data[x], data[y], marker=self.attr_sym, color=self.attr_col, markersize=self.attr_size,
                       linestyle='None')
        self.graph.getFigure().supxlabel(x)
        self.graph.getFigure().supylabel(y)
        self.graph.getFigure().tight_layout()
        self.graph.draw()

    def _format_label(self, x, y, step):
        return \
            f"<b>Step {step}:</b><br/>" \
            f"{x:.3f}, {y:.3f}<br/>" \
            f"Cost: {self.learner.j(np.array([x, y])):.5f}"


if __name__ == '__main__':
    from Orange.widgets.utils.widgetpreview import WidgetPreview  # since Orange 3.20.0

    WidgetPreview(Scaterplot).run(Orange.data.Table("iris"))
